from typing import List, Union
import pandas as pd
import numpy as np
from ladybug.datacollection import HourlyDiscontinuousCollection, HourlyContinuousCollection, \
    DailyCollection, MonthlyCollection, MonthlyPerHourCollection, BaseCollection
from ladybug.epw import EPW
from .extension_types.arraytype import LadybugArrayType

def dataframe_from_collections(
    data_collections: List[Union[HourlyDiscontinuousCollection, HourlyContinuousCollection, DailyCollection, MonthlyCollection, MonthlyPerHourCollection]],
    name_key: str = None,
) -> pd.DataFrame:
    """Generate a Dataframe from a list of ladybug data collections

    Example:
        .. code-block:: python
        import ladybug_pandas as lbp
        from ladybug.wea import Wea
        from ladybug.location import Location

        location = Location(
            city='Llanfairpwllgwyngyllgogerychwyrndrobwllllantysiliogogogoch',
            state='Wales',
            country='United Kingdom',
            latitude=53.224622,
            longitude=-4.197995,
            time_zone=0
        )

        wea = Wea.from_ashrae_clear_sky(location=location)

        df = lbp.dataframe_from_collections([
            wea.direct_normal_irradiance,
            wea.diffuse_horizontal_irradiance,
            wea.global_horizontal_irradiance,
            wea.direct_horizontal_irradiance,
        ])

    Arguments:
        data_collections {List[Union[HourlyDiscontinuousCollection, HourlyContinuousCollection, DailyCollection, MonthlyCollection, MonthlyPerHourCollection]]} -- A list of datacollections. Must all be of the same type

    Keyword Arguments:
        name_key {str} -- The name of the metadata key to use as a column name. Will default to datatype name if not specified (default: {None})

    Returns:
        pd.DataFrame -- A pandas dataframe. Each column will have a type of LadybugArrayType
    """
    
    data_collection_type = None

    df_list = []

    for collection in data_collections:
        # Check they are data collections
        assert isinstance(collection, BaseCollection), \
            f'All items of data_collections must be a type of ladybug data collection, not: {type(collection)}'

        # Assert same type of data collection
        if data_collection_type is None:
            data_collection_type = type(collection)
        else:
            assert isinstance(collection, data_collection_type), \
                f'All items of data_collections must be of the same type of data collection. Found {data_collection_type} and {type(collection)}'


        # Create columns from collections
        array = LadybugArrayType._from_data_collection(collection)

        if name_key is not None:
            col_name = collection.header.metadata[name_key]
        else:
            col_name = collection.header.data_type.name

        data = {}

        data[col_name] = array

        df_list.append(pd.DataFrame(
            data=data,
            index=collection.datetimes
        ))

    df = pd.concat(df_list, axis=1)

    return df


def dataframe_from_epw(
    epw: EPW
) -> pd.DataFrame:
    """Generate a Dataframe from an EPW object

    Example:
        .. code-block:: python

        import ladybug_pandas as lbp
        from ladybug.epw import EPW

        epw_path = 'tests/assets/epw/tokyo.epw'

        epw = EPW(epw_path)

        df = lbp.dataframe_from_epw(epw)

    Arguments:
        epw {EPW} -- A ladybug EPW object

    Returns:
        pd.DataFrame -- A pandas dataframe. Each column will have a type of LadybugArrayType
    """
    df = dataframe_from_collections(epw._data[6:])

    df = df.replace(999999999.0, np.nan)

    return df



import pandas as pd
from ..extension_types.arraytype import LadybugArrayType

@pd.api.extensions.register_series_accessor("ladybug")
class LadybugSeriesAccessor:

    def __init__(self, pandas_obj):
        self._obj = pandas_obj

    @property
    def is_lb_array(self):
        return isinstance(self._obj.values, LadybugArrayType)

    def check_type(self):
        assert self.is_lb_array, \
            f'Cannort perform ladybug operation on Array of type {type(self._obj.values)}'

    def to_si(self):
        self.check_type()
        return pd.Series(self._obj.values.to_si(), index=self._obj.index)

    def to_ip(self):
        self.check_type()
        return pd.Series(self._obj.values.to_ip(), index=self._obj.index)

    def to_unit(self, unit):
        self.check_type()
        return pd.Series(self._obj.values.to_unit(unit), index=self._obj.index)


@pd.api.extensions.register_dataframe_accessor("ladybug")
class LadybugDataFrameAccessor:

    def __init__(self, pandas_obj):
        self._obj = pandas_obj

    def to_si(self):
        return self._obj.apply(lambda x: x.ladybug.to_si() if x.ladybug.is_lb_array else x)

    def to_ip(self):
        return self._obj.apply(lambda x: x.ladybug.to_ip() if x.ladybug.is_lb_array else x)

    def to_unit(self, unit):
        return self._obj.apply(lambda x: x.ladybug.to_unit(unit) if x.ladybug.is_lb_array else x)



"""ladybug-pandas library."""

from .dataframe import dataframe_from_collections, dataframe_from_epw
from .series import series_from_collection

from .accessors.ladybug import LadybugSeriesAccessor, LadybugDataFrameAccessor
from .accessors.psychrometrics import PsychrometricsAccessor


import pandas as pd
from .extension_types.arraytype import LadybugArrayType

def series_from_collection(data_collection) -> pd.Series:
    """Generate a pandas Series from a Ladybug Data Collection

    Arguments:
        data_collection {ladybug._datacollectionbase.BaseCollection} -- An instance of a ladybug data collection

    Returns:
        pd.Series -- A pandas Series of type Ladybug Array
    """

    array = LadybugArrayType._from_data_collection(data_collection)

    return pd.Series(
        data=array,
        index=data_collection.datetimes
    )